#!/usr/bin/python
# hal.py - a class for working with HAL.
# Author: Will Woods <wwoods@redhat.com>

# NOTES: 
# This could probably be cleaner but it was really just designed for my own
# convenience. It could also use documentation on the interface for Device, or
# a wrapper class for that.

import dbus

class Hal:
    "A class for working with HAL."
    def __init__(self):
        self.bus = dbus.SystemBus()
        manager_obj = self.bus.get_object('org.freedesktop.Hal','/org/freedesktop/Hal/Manager')
        self.manager = dbus.Interface(manager_obj,'org.freedesktop.Hal.Manager')
    def GetDevice(self,udi):
        "Return a Device object for the given UDI."
        dev_obj=self.bus.get_object('org.freedesktop.Hal',udi)
        return dbus.Interface(dev_obj,'org.freedesktop.Hal.Device')
    def GetAllDevices(self):
        "Return a list of UDIs for all devices HAL knows about."
        return self.manager.GetAllDevices()
    def getDeviceProperties(self,udi):
        "Return a dict of property info for the device with the given udi."
        return self.GetDevice(udi).GetAllProperties()
    def devices(self):
        "Return a list containing property info dicts for all devices."
        return [self.getDeviceProperties(d) for d in self.GetAllDevices()]
    def dump(self,udi):
        "Return a string containing device properties for the given device"
        dev=self.getDeviceProperties(udi)
        rv="udi = %s\n" % udi
        for (k,v) in [(k,dev[k]) for k in sorted(dev.keys())]:
            if type(v) == type(list()):
                v=[i.encode("utf-8") for i in v]
            rv += "  %s = %s\n" % (k,v)
        return rv
    def dumpall(self):
        "Dump device properties for all devices (like lshal)"
        return "\n".join([self.dump(udi) for udi in self.GetAllDevices()])

if __name__ == "__main__":
    h=Hal()
    print h.dumpall()
