#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import string
import os
import sys

from hts.tags import Constants

class TestMetaData:
    
    FileName="testinfo.desc"
    
    def __init__(self, name):
        self.name = name # the simple name
        self.owner = ""
        self.nameSpace = ""
        self.testPackage = ""
        self.testName = "" # format: <namespace>/<name>
        self.path = ""
        self.version = ""
        self.description = ""
        self.testTime = ""
        self.runFor = ""
        self.runOn = ""
        self.requires = ""
        self.interactive = False
        self.priority = 999
        self.planUDI = ""
        self.planDeviceLabel = ""
        self.serverMode = False
        
    def __cmp__(self, other):
        if self.interactive and not other.interactive: return -1
        if other.interactive and not self.interactive: return 1
        if self.priority <= other.priority: return -1
        return 1
    
    def __str__(self):
        return self.name
    
    def __hash__(self):
        return hash(self.testName)

    def Owner(self):
        return self.owner
    def Name(self):
        return self.name
    def TestName(self):
        return self.testName
    def Path(self):
        return self.path
    def Version(self):
        return self.version
    def Description(self):
        return self.description
    def TestTime(self):
        return self.testTime
    def RunFor(self):
        return self.runFor
    def RunForDevice(self):
        return self.runOn
    def Interactive(self):
        if self.interactive: return Constants.yes
        return Constants.no
    def ServerMode(self):
        if self.serverMode: return Constants.yes
        return Constants.no
    def Priority(self):
        return ("%u" % self.priority)
    def Requires(self):
        return self.requires
    
    def setPlanUDI(self, udi):
        self.planUDI = udi
    def getPlanUDI(self):
        return self.planUDI
    def setPlanDeviceLabel(self, label):
        self.planDeviceLabel = label
    def getPlanDeviceLabel(self):
        return self.planDeviceLabel
    
    def write(self, filepath):
        file = open(filepath, "w")
        file.write("Owner: %s\n" % self.Owner())
        file.write("Name: %s\n" % self.TestName())
        file.write("Path: %s\n" % self.Path())
        file.write("Version: %s\n" % self.Version())
        file.write("Description: %s\n" % self.Description())
        file.write("TestTime: %s\n" % self.TestTime())
        file.write("RunFor: %s\n" % self.RunFor())
        file.write("Requires: %s\n" % self.Requires())
        file.write("X-RunForDevice: %s\n" % self.RunForDevice())
        file.write("X-Interactive: %s\n" % self.Interactive())
        file.write("X-Priority: %s\n" % self.Priority())
        file.write("X-ServerMode: %s\n" % self.ServerMode())
        file.close()
        
    def read(self, filepath):
        file = open(filepath, "r")
        for line in file:
            if '#' in line: line=line[0:line.index('#')]
            if "Owner:" in line: self.owner = line[line.index(':')+1:].strip()
            if "Name:" in line: self.testName = line[line.index(':')+1:].strip()
            if "Path:" in line: self.path = line[line.index(':')+1:].strip()
            if "Version:" in line: self.version = line[line.index(':')+1:].strip()
            if "Description:" in line: self.description = line[line.index(':')+1:].strip()
            if "TestTime:" in line: self.testTime = line[line.index(':')+1:].strip()
            if "RunFor:" in line: self.runFor = line[line.index(':')+1:].strip()
            if "Requires:" in line: self.requires = line[line.index(':')+1:].strip()
            if "X-RunForDevice:" in line: self.runOn = line[line.index(':')+1:].strip()
            if "X-Interactive:" in line: self.interactive = ( Constants.yes in line[line.index(':')+1:].strip())
            if "X-Priority:" in line: self.priority = string.atoi(line[line.index(':')+1:].strip())
            if "X-ServerMode:" in line: self.serverMode = ( Constants.yes in line[line.index(':')+1:].strip())
               
        file.close()
            
        
class Test:
    
    def __init__(self, name):
        self.metaData = TestMetaData(name)
        self.returnValue = 1
        self.result = Constants.ABORT
        
	try:
            self.outputFile = os.environ[Constants.OUTPUTFILE]
        except KeyError:
            self.outputFile = None
        
	try:
            self.udi = os.environ[Constants.UDI]
        except KeyError:
            self.udi = None
            
	try:
            self.mode = os.environ[Constants.RUNMODE]
        except KeyError:
            self.mode = None
            
	try:
            self.testServer = os.environ[Constants.TESTSERVER]
        except KeyError:
            self.testServer = None
        
            
    def getOutputFile(self):
        return self.outputFile
    def getUDI(self):
        return self.udi
    def getMode(self):
        return self.mode
    def getTestServer(self):
        return self.testServer
            
    
    def getMetaData(self):
        return self.metaData
    
    def initialize(self, args):
        pass
        
    def install(self):
        pass
    
    def run(self):
        pass
    
    def remove(self):
        pass
    
    def startServer(self):
        pass
    
    def stopServer(self):
        pass

    def statusServer(self):    # YK
        pass                   # YK
    
    def do(self, argv):
        try:
            if argv[1:][0] == "run":                
                return self.run()
            if argv[1:][0] == "metadata":
                self.getMetaData().write("testinfo.desc")
                return
        except IndexError:  # no args
            return self.run()
        
        try:
            if argv[1:][0] == "server":
                if argv[2:][0] == "stop":
                    self.stopServer();
                elif argv[2:][0] == "status":                          # YK
                    self.statusServer();                               # YK
                else:
                    self.startServer();
                return
            
        except IndexError:
            pass
        
        try:
            if argv[0] == "prompt":
                return self.promptConfirm(argv[1])           
        except IndexError:
            pass
        
        print "Error: bad test args"
        return
        
        
    
    # utilities
    def printPipe(self, pipe):
        while 1:
            line = pipe.readline()
            if line:
                print line
            else:
                return pipe.close()
            
    def removeDirectory(self, dir):
        "Remove a directory (and all its contents)"
        try:
            for (root,dirs,files) in os.walk(dir, topdown=False):
                for f in files: os.unlink(os.path.join(root,f))
                for d in dirs: os.rmdir(os.path.join(root,d))
                os.rmdir(dir)
        except OSError:
            return

    def prompt(self, question, answers=None):
        sys.stdout.write(question)
        if answers:
            sys.stdout.write(" (")
            first = True
            for answer in answers:
                if first:
                    sys.stdout.write(answer)
                    first = False
                else:
                    sys.stdout.write("|%s" % answer)
            sys.stdout.write(") ")
        sys.stdout.flush()
        response = sys.stdin.readline()
        if response.strip():
            sys.stdout.write("response: %s" % response)
        return response.strip()
    
    def promptConfirm(self, question):
        return self.prompt(question, ("yes", "no")) == "yes"   
    
    def promptContinue(self, message):
        self.prompt(question=message)
        return True
