#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Will Woods
#
import os
import re
from hts.tags import Constants
from hts.test import TestMetaData

class TestInfo:
    
    
    Debugging=False
    
    def __init__(self, testDirectory, debug=False):
        self.testExpressionMap=dict()
        self.testsByName=dict()
        self.generalTests = list()
        self.serverTests = list()
        self.testDirectory=testDirectory
        self.Debugging=debug
        if os.access(testDirectory,os.R_OK|os.X_OK):
            self.read()

    def read(self):
        "Walk through the test dir and read all the metadata we know about"
        if self.Debugging:
            print "TestInfo.read: searching test metadata"
        for (root,dirs,files) in os.walk(self.testDirectory):
            if TestMetaData.FileName in files:
                name=root[root.rindex('/')+1:]
                testMetaData = TestMetaData(name)
                testMetaData.read(root+'/'+ TestMetaData.FileName)
                if testMetaData.RunForDevice() == Constants.GENERAL:
                    self.generalTests.append(testMetaData)
                    if self.Debugging:
                        print "    General Test: %s Expression: \'%s\'" % (testMetaData.Name(), testMetaData.RunForDevice())
                else:
                    self.testExpressionMap[testMetaData]=testMetaData.RunForDevice()
                    if self.Debugging:
                        print "    Device Test: %s Expression: \'%s\'" % (testMetaData.Name(), testMetaData.RunForDevice())
                
                if testMetaData.ServerMode() in Constants.yes:
                    if self.Debugging:
                        print "    Server Mode Test: %s" % testMetaData.Name()
                    self.serverTests.append(testMetaData)
                self.testsByName[name] = testMetaData
        
        if self.Debugging:
            print "...Done."

    def getTests(self, props):
        """Return a list of test names that match this device.
        We check this by evaluating the expressions in the hwinfo.desc files for
        each test, replacing tokens that look like $some.prop with the appropriate
        HAL property from the given device.
        """
        if self.Debugging:
            print "TestInfo.getTests: "
            print props
        tests=list()
        

        def repl(m):
            if m.group(1) == "''":
                return "'"
            if m.group(1) in props:
                return "props['%s']" % m.group(1)          
            #otherwise
            return "'%s'" % m.group(1) 
            
        
        for (test,expr) in self.testExpressionMap.iteritems():
            parsed_expr = re.sub('([A-Za-z0-9_]+\.[A-Za-z0-9_.]+)',repl,expr)
            parsed_expr = re.sub('(\'\')',repl,parsed_expr)
            # FIXME: security risk! This eval needs to be safened!
            if self.Debugging:
                print "parsed expression: \'%s\'" % (parsed_expr)
            if eval(parsed_expr):
                tests.append(test)
                if self.Debugging:
                    print "   added: %s" % (test)
        if self.Debugging:
                print "...Done."
        return tests
    
    def getGeneralTests(self):
        return self.generalTests
    
    def getServerModeTests(self):
        return self.serverTests
    
    def getTest(self, name):
        return self.testsByName[name]
