#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Ported by : YangKun <ykun@redhat.com>
#


# find the floppy devices      -YangKun
get_floppy_info() {
    hal-device |  fgrep linux.sysfs_path_device | awk '{print $3}' | grep floppy | tr -d "'.""\/"[a-z] | sort -u
}


floppy_test() {
    # if there are no floppy found	-YangKun
    FLOPPY_DEVICES=`get_floppy_info`
    if [ -z $FLOPPY_DEVICES ]; then
      echo "Error! No floppy device found !"
      return 1
    else
      echo "Find floppy fd$FLOPPY_DEVICES, will use it to do the test !"
    fi

    # Some machines report detached floppy drives.
    # remove this "for" loop ?	-YangKun
    for device in $FLOPPY_DEVICES
    do
      sed 's/^-$/%/' /etc/sysconfig/hwconf |while read -d% chunk;do echo $chunk;done |
      grep "device: fd$device\b"|grep -q 'detached: 1' || echo "fd$device"
    done

    # Create the test images
    mkdir -p ./image ./device
    dd if=/dev/zero of=./floppy.img bs=1k count=1440
    mkdosfs ./floppy.img
    mount -o loop ./floppy.img ./image
    cp -a /etc/* ./image 2> /dev/null
    umount ./image


    # run the test !	-YangKun
    for floppy in $FLOPPY_DEVICES
    do
      # if ! mount /dev/fd$floppy ./device 2>/dev/null; then
      if ! dd bs=1c if=/dev/fd$floppy count=0 2>/dev/null; then
          clear
          echo "Please insert a blank floppy disc into floppy "
          echo "device fd$floppy and then press ENTER to continue !"
          read
      fi
      if ! dd bs=1c if=/dev/fd$floppy count=0 2>/dev/null; then
        echo "Error ! No floppy disc or bad media in fd$floppy !"
        return 1
      fi
      dd if=./floppy.img of=/dev/fd$floppy bs=1k count=1440
      mount /dev/fd$floppy ./device
      mount -o loop ./floppy.img ./image
      diff -r ./device ./image
      umount ./image ./device
    done

    # remove temp files	-YangKun
    rm ./floppy.img
    rmdir ./device
    rm -rf ./image
}

# run the test ! 	-YangKun
floppy_test


