#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os
import tempfile
import string
import signal
import sys
import re
import time

# sys.path.append("/home/gnichols/devel/hts")
from hts.hal import Hal
from hts.test import Test

class InfoTest(Test):        
    
    def __init__(self):
        Test.__init__(self, name="info")
        
        self.getMetaData().owner = "Greg Nichols <gnichols@redhat.com>"
        self.getMetaData().nameSpace = "HTS"
        self.getMetaData().testPackage = "hts"
        self.getMetaData().testName = ""
        self.getMetaData().path = "hts/tests/info"
        self.getMetaData().version = "1.1"
        self.getMetaData().description = "Hardware Certification, test to gather system information."
        self.getMetaData().testTime = "5m"
        self.getMetaData().runFor = ""
        self.getMetaData().runOn = "General"
        self.getMetaData().requires = "hts"

    def checkKernel(self):
        pipe = os.popen("uname -r | sed -re 's/([^a-z]*)(smp|hugemem|largesmp)$/\2-\1/'")
        kv = ""
        kernelrpm = ""
        while 1:
           line = pipe.readline()
           if line: 
               kv=line
               kernelrpm = "kernel-%s" % kv
           else:  break
        pipe.close()
        
        print "checking kernel %s" % kernelrpm
         
        fail = False
  
        if os.system("grep -q 0 /proc/sys/kernel/tainted") != 0:
            print "Your kernel is tainted; please remove the non-GPL modules."
            fail = True
        else:
            print "The kernel is untainted"
            
  
        # check for third-party kernel modules
        pipe = os.popen("rpm -ql %s" % kernelrpm)
        kernelModules = list()
        while 1:
           line = pipe.readline()
           if line: 
               kernelModules.append(line)
           else:  break
        pipe.close()
        
        procModules = list()
        procModulesFile = open("/proc/modules", "r")
        while 1:
            line = procModulesFile.readline()
            if line:
                module = line.split(" ", 1)
                procModules.append(module)
            else:
                break;
        procModulesFile.close()
        
        for procModule in procModules:
            pattern = re.compile("/%s\.k?o$" % procModule)
            for kernelModule in kernelModules:
                match = pattern.search(kernelModule)
                if match and match.group():
                    print "Non-Red Hat kernel module found: %s" % procModule
                    fail = True
                    break
            
        if fail:
            print "Error: tainted kernel found."
            return False
        
        # otherwise
        print "No non-Red Hat kernel modules were found."
            
        if os.system("rpm -V --nofiles %s" % kernelrpm) == 0:
            print "Error: This kernel RPM was not signed by Red Hat!"
            return False
        
        # otherwise
        print "Kernel RPM was signed by Red Hat"
        
        if os.system("rpm -V --nomtime --nomode --nocontexts %s" % kernelrpm) == 0:
            print "Error: files from %s were modified" % kernelrpm
            return False
        
        #otherwise
        print "No modified files in kernel RPM"
        
        if os.system("rpm -q %s --qf \"%%{buildhost}\n\" | grep -q redhat.com" % kernelrpm) == 0:
            print "Error: This kernel was not built by Red Hat!" 
            return False
    
        # otherwise
        print "Kernel Checks Passed!"
        return True # passed


    def run(self):
        
        returnValue = self.checkKernel()
        if returnValue != 0:
            return returnValue;
        
        # GBN: the rpm part just seems to hang - I've disabled it for now
        sysreportCommand = "sysreport -norpm"

        attachmentDirectory = "."
        tarfile = None
        
        pipe = os.popen("echo -e '\n\n' | sysreport -norpm 2>&1")
        while 1:
            line = pipe.readline()
            if line:
                if "Please send" in line:
                    tarball = line.split(" ", 3)
            else: break
            
        if tarfile:
            shutil.copy(tarball, attachmentDirectory)
            
        try:
            os.system("/usr/sbin/dmidecode")
        except:
            pass # not a failure if dmidecode is not there, or doesn't work

        return 0; # passes

if __name__ == "__main__":
    
    test = InfoTest()
    test.do(sys.argv)
   
   


