#!/bin/bash
# Copyright (C) 2005 Red Hat, Inc. All Rights Reserved.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# (automated) testname: INFO

hardware_INFO(){
 find_rhn_hardware class
}

check_kernel() {
	
    local kv=$(uname -r | sed -re 's/([^a-z]*)(el5)(xen)$/\3-\1\2/')
    local kernelrpm=kernel-$kv
    
    local fail=false
    # check to see if the kernel is tainted
    if ! grep -q 0 /proc/sys/kernel/tainted; then
        echo "Your kernel is tainted; please remove the non-GPL modules."
        fail=true
    fi
    # check for third-party kernel modules
    rpm -ql $kernelrpm | tr _ - > $kernelrpm.modlist

    # add the following line to be compatible with HwCert website catalog   --YK
    echo "+ rpm -ql $kernelrpm"
                                     
    for module in $(awk '{print $1}' /proc/modules | tr _ -); do
        if ! egrep -q "/$module\.k?o$" $kernelrpm.modlist; then
            echo "Non-Red Hat kernel module found: $module"
            fail=true
        fi
    done
    if $fail; then echo "Error: tainted kernel found."; return 1; fi
    
    # check the kernel RPM signature (not the files themselves)
    if ! rpm -V --nofiles $kernelrpm; then 
        echo "Error: This kernel RPM was not signed by Red Hat!"; return 1
    fi

    # check the integrity of the kernel RPM contents.
    # NOTE: we skip the mtime, mode, and SELinux context tests here because:
    # a) The ia64 kernel needs to be installed onto a vfat partition, which 
    #    does not properly store these properties, so 'rpm -V kernel' always
    #    fails on ia64 (see bug #144712)
    # b) We don't really care if the permissions are correct, just that the
    #    file contents have not been changed. We're still checking size and
    #    md5sum and that should be good enough.
    if [ ${kv:0:3} == "2.6" ]; then local selinux='true'; fi
    if ! rpm -V --nomtime --nomode ${selinux:+--nocontexts} $kernelrpm; then 
        echo "Error: files from $kernelrpm were modified"; return 1
    fi
    
    # and check the buildhost
    if ! rpm -q $kernelrpm --qf "%{buildhost}\n" | grep -q redhat.com; then
        echo "Error: This kernel was not built by Red Hat!"; return 1
    fi
}

automated_INFO(){
# GBN: the rpm part just seems to hang - I've disabled it for now

logdir=$(dirname $OUTPUTFILE)
echo "logdir: $logdir"

# At this point we cheat and just call sysreport...
echo -e '\n\n' | sysreport -norpm 2>&1 | tee sysreport.log
tarball=$(grep "^Please send" sysreport.log | awk '{ print $3; }')
cp $tarball $logdir


check_kernel
case $(uname -i) in
    i386|x86_64) [ -x /usr/sbin/dmidecode ] && [ !$(hal-device | fgrep -q xen) ] && /usr/sbin/dmidecode ;;
esac

exit 0
}



automated_INFO

