#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os
import tempfile
import string
import signal
import sys
import re
import time

# sys.path.append("/home/gnichols/devel/hts")
from hts.tags import Constants
from hts.hal import Hal
from hts.test import Test

class NetworkTest(Test):        
    
    def __init__(self):
        Test.__init__(self, name="network")
        self.ipaddr = ""
        self.testFile = "/var/www/html/httptest.file"
        self.allInterfaces = list()
        self.interfaceSpeed = 100 # speed in Mb/sec
        self.oldHandler = None
        
        self.getMetaData().owner = "Greg Nichols <gnichols@redhat.com>"
        self.getMetaData().nameSpace = "HTS"
        self.getMetaData().testPackage = "hts"
        self.getMetaData().testName = ""
        self.getMetaData().path = "hts/tests/network"
        self.getMetaData().version = "1.1"
        self.getMetaData().description = "Network IF test for tcp, udp, icmp and nfs"
        self.getMetaData().testTime = "5m"
        self.getMetaData().runFor = ""
        self.getMetaData().runOn = "\'net\'  in info.capabilities"
        self.getMetaData().requires = "lmbench"
        self.getMetaData().serverMode = True

    def prepareForRun(self):
        hal = Hal()
        self.deviceProperties =  hal.getDeviceProperties(self.getUDI())
        self.interface = self.deviceProperties['net.interface']
 

   
    def nfsSignalHandler(self, sig, frame):
        try:
            removeDirectory("%s/%s" % (self.nfsdir, self.ipaddr))
            os.system("umount %s" % self.nfsdir)
            os.rmdir(self.nfsdir)
            print 'signal caught; cleaning up NFS mount'
        except:
            return
        
    def setSignalHandler(self, handler):
        signals = [signal.SIGINT, signal.SIGTERM]
        for sig in signals:
            if handler:
                self.oldHandler = signal.signal(sig, handler)
            else:
                signal.signal(sig, self.oldHandler)
            
    def downAllInterfaces(self):
        print "Shutting down all interfaces..."
        self.allInterfaces = list()
        pipe = os.popen("/sbin/ip -o link show | awk -F \": \" '/UP>/ { print $2 }' | grep -v lo")
        while 1:
            line = pipe.readline()
            if line:
                print "    interface: %s" % line
                if os.system("ifdown %s" % line):
                    self.allInterfaces.append(line)
                # confirm that it really went down
                slept=0
                delay=15
                while slept < delay:
                    if not os.system("ifconfig %s | fgrep \"RUNNING\" -q" % self.interface):
                        time.sleep(1)
                        slept = slept + 1
                    else:
                        print "Interface %s is down" % self.interface
                        break
                if slept > delay:
                    print "Unable to shut down interface %s" % self.interface
            else:
                break
        pipe.close()
        print "waiting..."
        time.sleep(10)
        print "done"
       
        
        
    def restoreAllInterfaces(self, sig=None, frame=None):
        print "Restoring all interfaces..."
        for interface in self.allInterfaces:
            print "    interface: %s" % interface
            os.system("ifup %s" % interface)
        
    def bringUpInterface(self):
        print "Bringing up interface %s" % self.interface
        slept=0
        delay=15
        os.system("ifup %s" % self.interface)
        print "Checking..."
        # Since there can be a delay between ifup and linkup we need to
        # wait to see the interface running.
        while slept < delay:
           if os.system("ifconfig %s | fgrep \"RUNNING\" -q" % self.interface):
               time.sleep(1)
               slept = slept + 1
           else:
               print("Interface %s is RUNNING" % self.interface)
               break
       
        if slept > delay:
            os.system("ifdown %s" % self.interface)
            return False;

        # The next piece of bad news is that not all drivers support "RUNNING"
        # properly so we also need to see if we can reach the other end each time.
        # After a possible 30 seconds total we give up.
        slept=0
        delay=15
        print("Checking via ping to %s..." % self.getTestServer())
        while slept < delay:
            if os.system("ping -Ac1 %s > /dev/null" % self.getTestServer()):
                time.sleep(1)
                slept = slept + 1
            else:
                print "  done"
                break
            
        if slept > delay:
            print "FAILED, could not ping %s" % self.getTestServer
            return False
        
        # otherwise, it came up, get the ip address of the interface
        # what about IP v6?
        pipe = os.popen("ip -4 addr show %s" % (self.interface))
        pattern = re.compile("\d+\.\d+\.\d+\.\d+")
        while 1:
            line = pipe.readline()
            match = pattern.search(line)
            if match or not line:
                break
        pipe.close()
        if match:
            self.ipaddr = match.group()
            print "ip address: %s" % self.ipaddr
            return True
    
    def createTestFile(self):
        try:
            os.remove(self.testFile)
        except OSError:
            pass
            
        self.getInterfaceSpeed()
        returnValue = os.system("dd if=/dev/urandom of=%s bs=128k count=%s" % (self.testFile, self.interfaceSpeed))
        return returnValue == 0
    
    def getInterfaceSpeed(self):       
        for interfaceString in (self.interface, "p%s" % self.interface):
            ethtoolCommand = "ethtool %s | fgrep \"Speed\"" % interfaceString
            pipe = os.popen(ethtoolCommand)
            line = pipe.readline()
            pipe.close()
            if line:
                pattern = re.compile("\d+")
                match = pattern.search(line)
                if match:
                    self.interfaceSpeed = string.atoi(match.group())
                    return
        # otherwise
        self.interaceSpeed = 100
        print "interface speed is %u" % self.interfaceSpeed

        
    # icmpTest uses ping with various packet sizes
    def icmpTest(self):
        packetSizes = [0, 1, 240, 256, 1024, 2000, 10000, 40000, 65507]
        for size in packetSizes:
            sizeParam = ""
            if size > 0:
                sizeParam = "-s %s" % size
            (input, pingPipe) = os.popen4("/bin/ping -f -c 1000 %s %s" % (sizeParam, self.getTestServer()))     
            returnValue = self.printPipe(pingPipe)
            if returnValue:
                return returnValue
        
        # otherwise
        return 0

    def tcpTest(self):
        print "tcp test:"
        cycles = 5
        for p in range(cycles):
            print "test cycle %u of %u" % (p+1, cycles)
            print "testing latency..."
            returnValue = os.system("lat_tcp %s" % self.getTestServer())
            if returnValue != 0:
                print "FAILED %u" % returnValue
                return returnValue
            
            # bw_tcp seems to return non-zero for success, and zero
            # for failure?
            print "testing bandwidth to %s..." % self.getTestServer()
            returnValue = os.system("bw_tcp %s 1m" % self.getTestServer())
            if returnValue == 0:
                print "FAILED (%s)" % returnValue    
                return 1
        print "passed"
        return 0

    def startServer(self):
        # start the bandwidth test server
        os.system("bw_tcp -s")
        os.system("lat_udp -s")
        # nfs
        os.system("service nfs start")
        try:
            os.mkdir("/var/hts")
        except OSError: # file probably exists
            pass
        try:
            os.mkdir("/var/hts/export")
        except OSError: # file probably exists
            pass
        
        os.system("exportfs -o rw,no_root_squash *:/var/hts/export")
        # httpd
        os.system("service httpd start")
        
    def stopServer(self):
        os.system("bw_tcp -S localhost")
        os.system("lat_udp -S localhost")            
        # nfs
        os.system("service nfs stop")
        # httpd
        os.system("service httpd start")

    def statusServer(self):                                                     # YK
        # TODO: what should we check here ?                                     # YK
        os.system("exportfs | grep /var/hts/export") == "/var/hts/export "      # YK
        os.system("service nfs status ")                                    # YK
        # httpd
        os.system("service httpd status")
        
    def udpTest(self):
        print "testing udp latency to %s..." % self.getTestServer()
        (input, output) = os.popen4("lat_udp %s" % self.getTestServer())
        line = output.readline()
        print line.strip()
        if line.find("failed") >= 0:
            print "FAILED"         
            return 1
        #otherwise
        print "passed"
        return 0


    def nfsTest(self):
        print "testing nfs using %s..." % self.getTestServer()
        self.nfsdir=tempfile.mktemp("nfsdir")
        os.system("mkdir -p %s" % self.nfsdir)
        files=10
        nfsopts="rw,intr,rsize=32768,wsize=32768,udp"
        
        print "Copying %s x %u mbit files to/from %s" % (files, self.interfaceSpeed, self.getTestServer())
        
        txtime=0 
        rxtime=0 
        t=0
        mbit=files*self.interfaceSpeed # where does speed come from?
        
        # Mount NFS dir and create a test subdir for this machine
        mountCommand = "mount -o %s %s:/var/hts/export %s" % (nfsopts, self.getTestServer(), self.nfsdir)
        print mountCommand
        returnValue = os.system(mountCommand)
        if returnValue != 0:
            print "Mount command FAILED - returned %s" % returnValue
            return returnValue
        
        self.setSignalHandler(self.nfsSignalHandler)
        os.system("rm -rf %s/%s" % (self.nfsdir, self.ipaddr))
        os.system("mkdir -p %s/%s" % (self.nfsdir, self.ipaddr))
        
        # write test: copy random data to remote machine a buncha times
        for n in range(files):
            print "copy from  %s to %s/%s/%u"  % (
                          self.testFile, 
                          self.nfsdir, self.ipaddr, n)

            os.system("/usr/bin/time -f%%e cp %s %s/%s/%u 2>time.tmp" % (self.testFile, self.nfsdir, self.ipaddr, n))
            timeFile = file("time.tmp", "r")
            time = timeFile.readline()
            txtime = txtime + string.atof(time)
            timeFile.close()
        self.setSignalHandler(None)
        os.system("umount %s" % self.nfsdir)
        rate = 0.0
        if txtime > 0:
            rate = mbit/txtime
        print "%u mbit transmitted in %u sec ( %e mbit/s)" % (mbit, txtime, rate)
        # read test: copy data from remote machine (and compare it to original)
        returnValue = os.system("mount -o %s %s:/var/hts/export %s" % (nfsopts, self.getTestServer(), self.nfsdir))
        if returnValue != 0:
            print "Mount command FAILED - returned %s" % returnValue
            return returnValue
        
        self.setSignalHandler(self.nfsSignalHandler)
        
        for n in range(files):
            print "copy from  %s/%s/%u to %s"  % (
                          self.nfsdir, self.ipaddr, n,
                          self.testFile) 
            os.system("/usr/bin/time -f%%e cp %s/%s/%u datafile.tmp 2>time.tmp" % (self.nfsdir, self.ipaddr, n))
            os.system("cmp %s datafile.tmp" % self.testFile)
            os.remove("%s/%s/%u" % (self.nfsdir, self.ipaddr, n))
            os.remove("datafile.tmp")
            timeFile = file("time.tmp", "r")
            time = timeFile.readline()
            rxtime = rxtime + string.atof(time)
            timeFile.close()
            
        self.setSignalHandler(None)
        os.system("umount %s" % self.nfsdir)
        print "%u mbit received in %u sec ( %e mbit/s)" % (mbit, rxtime, mbit/rxtime)
        return 0 # success

    def run(self):       
        if self.getTestServer() == None:
            print "Error: No test server was set, test abort"
            return 1
            
        self.prepareForRun()
        print "using test server %s" % self.getTestServer()
        returnValue = None
        self.downAllInterfaces()
        self.setSignalHandler(self.restoreAllInterfaces)
        
        if not self.bringUpInterface():
            returnValue = 1
            print ("Interface %s did not come up" % self.interface)
        elif not self.createTestFile():
            returnValue = 1
            print ("Unable to create test file")
        else:
            returnValue = self.tcpTest()
            if returnValue ==0:
                returnValue = self.udpTest()
            if returnValue ==0:
                returnValue = self.nfsTest()
            if  returnValue==0:
                returnValue = self.icmpTest()
           
        self.setSignalHandler(None)
        self.restoreAllInterfaces()
        
        if returnValue == 0:
            result = "passed"
        else:
            result = "failed"
        
        print "Network test on device %s %s" % (self.interface, result)
        
        return returnValue

if __name__ == "__main__":
    
    networkTest = NetworkTest()
    returnValue = networkTest.do(sys.argv)
    sys.exit(returnValue)
   
   


