#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os
import tempfile
import string
import signal
import sys
import re
import time
import dbus

# sys.path.append("/home/gnichols/devel/hts")
from hts.tags import Constants
from hts.hal import Hal
from hts.test import Test

class USBTest(Test):        
    
    def __init__(self):
        Test.__init__(self, name="usb")
        
        # metadata AKA testinfo.desc
        self.getMetaData().owner = "Greg Nichols <gnichols@redhat.com>"
        self.getMetaData().nameSpace = "HTS"
        self.getMetaData().testPackage = "hts"
        self.getMetaData().testName = ""
        self.getMetaData().path = "hts/tests/usb"
        self.getMetaData().version = "1.1"
        self.getMetaData().description = "USB Device/Port test"
        self.getMetaData().testTime = "5m"
        self.getMetaData().runFor = ""
        # was: "\'usb\'  == info.bus and \'hub\' == info.linux.driver"
        # change to run once on what ever USB is there.
        self.getMetaData().runOn = "info.product == 'Computer'"
        self.getMetaData().interacive = True
        self.getMetaData().requires = ""
        self.hal = Hal()
        
    def getBusAndPortNumbers(self, device):
        if device['info.bus'] != 'usb_device':
            return None
         
        deviceString  = device['linux.sysfs_path_device']
        pattern = re.compile("\/\d+-\d+")
        match = pattern.search(device['linux.sysfs_path_device'])
        if match: 
            dashPosition = string.find(match.group(), "-")
            busNumber = match.group()[1:dashPosition]
            portNumber = match.group()[dashPosition+1:]
            return (busNumber, portNumber, "%s-%s" % (busNumber, portNumber))
        return None
        
        
    def checkAllPortsUnplugged(self):
        devices = self.hal.devices()
        self.pluggedInDevices = list()
        returnValue = True
        for dev in devices:
             try:
                 found = self.getBusAndPortNumbers(dev)
                 if found:
                     (busNumber, portNumber, busDashPort) = found
                     print "%s %s appears to be plugged into bus number %s port number %s" %(dev['info.vendor'], dev['info.product'], busNumber, portNumber)
                     self.pluggedInDevices.append(busDashPort)
                     returnValue = False
             except KeyError:
                 pass
         
        return returnValue
             
    def addNewPluggedPort(self):
        devices = self.hal.devices()
        for dev in devices:
             try:
                 found = self.getBusAndPortNumbers(dev)
                 if found:
                     (busNumber, portNumber, busDashPort) = found
                     if busDashPort not in self.pluggedInDevices:
                         print "%s %s appears to be plugged into bus number %s port number %s" %(dev['info.vendor'], dev['info.product'], busNumber, portNumber)
                         return (busNumber, portNumber, busDashPort)
             except KeyError:
                 pass
        return None
        
    def confirmPortUnplugged(self, searchBusDashPort):
        devices = self.hal.devices()
        for dev in devices:
             try:
                 found = self.getBusAndPortNumbers(dev)
                 if found:
                     (busNumber, portNumber, busDashPort) = found
                     if busDashPort == searchBusDashPort:
                         return False
             except KeyError:
                 pass
        return True
 

    def run(self):
        # ask user for the number of ports:
        print "USB test:"
        self.numberOfSockets = string.atoi(self.prompt("How many unused USB sockets are there? "))
        if self.numberOfSockets < 1:
            print("No USB sockets to test")
            self.result = Constants.PASS
            return 0

        
        # otherwise
        self.pluggedPorts = list()
        # check for USB devices, print info about them
        self.checkAllPortsUnplugged()
        self.fixedDevices = len(self.pluggedInDevices)
        while len(self.pluggedInDevices)-self.fixedDevices < self.numberOfSockets:
            print "testing socket %s of %s..." % (len(self.pluggedInDevices)-self.fixedDevices+1, self.numberOfSockets)
            if self.promptConfirm("Please plug in a USB device - continue?"):
                
                # find out which port was plugged
                found = self.addNewPluggedPort()            
                if found:
                    (busNumber, portNumber, busDashPort) = found
                    print "found device at %s" % busDashPort
                    if self.promptContinue("Please unplug the device and hit the Enter key: "):
                        if self.confirmPortUnplugged(busDashPort):
                            print "confirmed device %s" % busDashPort
                            self.pluggedInDevices.append(busDashPort)
                        else:
                            print "Did not confirm the device - repeating test."
                else:
                    print "device not found"
            else:
                self.returnValue = 1
                self.result = Constants.ABORT
                return self.returnValue
                       
        if len(self.pluggedInDevices)-self.fixedDevices == self.numberOfSockets:
            self.returnValue = 0
            self.result = Constants.PASS
        else:
            self.returnValue = 1
            self.result = Constants.FAIL
                
        
        return self.returnValue

if __name__ == "__main__":
    
    usbTest = USBTest()
    # usbTest.udi = "/org/freedesktop/Hal/devices/usb_device_0_0_0000_00_1d_2_if0"
    # usbTest.run()
    returnValue = usbTest.do(sys.argv)
    sys.exit(returnValue)
   
   


