#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.#
# Author: Irina Boverman 
#

import os
import sys
import string
import commands
import shutil

 # sys.path.append("/home/gnichols/devel/hts")
from hts.test import Test

class VideoTest(Test):
# A wrapper for x11perf; while we're not actually looking for benchmark numbers
# we may perhaps one day start to compare and contrast against them to ensure
# there are no performance regressions.  Until then; we're using x11perf's
# use of X primitives to check for proper operation.  

    def __init__(self):
        Test.__init__(self, "video")
        
        self.Xconfig_tool = ""
        self.Xconfig_flag = ""
        self.depth = 0
        self.interface = "video-card"
        self.Xconfigfile = "/tmp/htsXconfig"
        self.tmpFile = "/tmp/hts.x11perf"
         
        self.getMetaData().owner = "Greg Nichols <gnichols@redhat.com>"
        self.getMetaData().nameSpace = "HTS"
        self.getMetaData().testPackage = "hts"
        self.getMetaData().testName = "video"
        self.getMetaData().path = "hts/tests/video"
        self.getMetaData().version = "1.1"
        self.getMetaData().description = "Video test: tests one video card in a time"

        self.getMetaData().testTime = "5m"
        self.getMetaData().runFor = ""
        self.getMetaData().runOn = "pci.device_subclass == 0  and  pci.device_class == 3 "
        self.getMetaData().requires = ""

    def settool(self):
        # Adapt for tool name changes
        (status, self.Xconfig_tool) = commands.getstatusoutput("rpm -q system-config-display > /dev/null && " +
                                                               "echo system-config-display || " +
                                                               "echo redhat-config-xfree86")
        if status != 0:
            print "FAILED video setup" 
        return status

    def getfile(self):
        status = os.system(self.Xconfig_tool + " -o " + self.Xconfigfile + "  --noui > /dev/null") 
        if status != 0:
            print "FAILED to obtain config file"
        return status

    def setdepth(self):
        (status, self.depth) = commands.getstatusoutput("grep DefaultDepth " + self.Xconfigfile + 
                                                        " | awk ' { print $2; } '")
        if status != 0:
            print "FAILED to obtain DefaultDepth"
            return status
        try:
            self.depth = int(self.depth)
        except:
            print "FAILED to obtain depth"
            status = 1            
        return status
    
    def setflag(self):
        # Adapt for flag changes
        (status, self.Xconfig_flag) = commands.getstatusoutput("X --help 2>&1 | grep xf86config -q && " +
                                                               "echo -xf86config || echo -config")
        if status != 0:
            print "FAILED to obtain config flag"    
        return status
         
    def startx(self):        
        try: os.remove(self.tmpFile)
        except: None

        status =  os.system("startx ./x11perftest -- :2 %s %s -depth %s &> /dev/null" % ( 
                             self.Xconfig_flag, self.Xconfigfile,
                             str(self.depth) 
                             ))
        # copy the x11perf log into the output file
        shutil.copy("/var/log/Xorg.0.log", ".")
        shutil.copy("/var/log/Xorg.2.log", ".")
        for filename in os.listdir("."):
            if "Xorg" in filename:
                xlog = open(filename)
                while 1:
                    line = xlog.readline()
                    if line: 
                        print line
                    else: 
                        break
                xlog.close();
                    
        
        if status != 0: 
            print "FAILED to execute startx"
        return status

    def xstatus(self):
        status = os.system("grep -q 0 %s" % self.tmpFile)
        if status != 0:
            print "FAILED xstatus command"
        return status
    
    def run(self):
        print "Starting video test"
	status = 1
        result = "FAILED"
        if (self.settool() == 0 and self.getfile() == 0 and 
            self.setdepth() == 0 and self.setflag() == 0 and 
            self.startx() == 0 and self.xstatus() == 0):
            status = 0
            result = "passed"
        print "Video test on device %s %s" % (self.interface, result) 
        return status

if __name__ == "__main__":
    videoTest = VideoTest()
    returnValue = videoTest.do(sys.argv)  
    sys.exit(returnValue)
