#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Name   : HW Cert virtualization test
# Author : YangKun <ykun@redhat.com>
#

# create a 6G file as the hard disk for Guest OS
create_hd() {
  echo "  Creating Guest's hard drive file ... "
  rm -rf $IMAGE_DIR/virt1.img
  dd if=/dev/zero of=$IMAGE_DIR/virt1.img bs=1024k count=6000
}

# create the kickstart cfg file for the FV Guest
create_ks_file() {
  echo "text"                                                  > $KS_FILE
  # can not skip the installation "key"
  # for RHEL5 Client, the key is CW, for RHEL5 Server, the key is CS
  echo "key skip"                                               >> $KS_FILE
  echo "keyboard us"                                          >> $KS_FILE
  echo "lang en_US"                                           >> $KS_FILE
  echo "langsupport --default en_US en_US"                    >> $KS_FILE
  # in Guest , the first NIC will always be eth0
  echo "network --device eth0 --bootproto dhcp"               >> $KS_FILE
  echo "nfs --server=$HOST_IP --dir=$INSTALL_MEDIA"           >> $KS_FILE
  echo "rootpw 111111"                                        >> $KS_FILE
  echo "skipx"                                                >> $KS_FILE
  echo "firewall --disabled"                                  >> $KS_FILE
  echo "authconfig --enableshadow --enablemd5"                >> $KS_FILE
  echo "selinux --disabled"                                   >> $KS_FILE
#  echo "selinux --enforcing"                                  >> $KS_FILE
  echo "timezone Asia/Shanghai"                               >> $KS_FILE
  echo "bootloader"                                           >> $KS_FILE
  echo "clearpart --initlabel"                                >> $KS_FILE
  echo "autopart"                                             >> $KS_FILE
  echo "firstboot --disable"                                  >> $KS_FILE
  echo "halt"                                                 >> $KS_FILE
  echo ""                                                     >> $KS_FILE
  echo "%packages"                                            >> $KS_FILE
  echo "@admin-tools"                                         >> $KS_FILE
  echo "@base"                                                >> $KS_FILE
  echo "@base-x"                                              >> $KS_FILE
  echo "@core"                                                >> $KS_FILE
  echo "@development-libs"                                    >> $KS_FILE
  echo "@development-tools"                                   >> $KS_FILE
  echo "@editors"                                             >> $KS_FILE
  echo "@gnome-desktop"                                       >> $KS_FILE
  echo "@gnome-software-development"                          >> $KS_FILE
  echo "@text-internet"                                       >> $KS_FILE
  echo "@web-server"                                          >> $KS_FILE
  echo "@x-software-development"                              >> $KS_FILE
  echo "PyXML"                                                >> $KS_FILE
  echo "elfutils-libs"                                        >> $KS_FILE
  echo "elfutils"                                             >> $KS_FILE
  echo "rpm-build"                                            >> $KS_FILE
  echo ""                                                     >> $KS_FILE
  echo "%post"                                                >> $KS_FILE
  # create a vtsh script to store our command
  # mount the Host's NFS dir to Guest's local dir
  echo "touch                                        $VT_SH"  >> $KS_FILE
  echo "chmod 755                                    $VT_SH"  >> $KS_FILE
  echo "echo '#!/bin/bash'                         > $VT_SH"  >> $KS_FILE
  echo "echo 'rm -rf /export'                     >> $VT_SH"  >> $KS_FILE
  echo "echo 'mkdir /export'                      >> $VT_SH"  >> $KS_FILE
  echo "echo 'mount $HOST_IP:$EXPORT /export'     >> $VT_SH"  >> $KS_FILE

  # install hts and related rpms on Guest OS
  echo "echo 'rpm -ivh /export/rpm/*.rpm'         >> $VT_SH"  >> $KS_FILE

  # create the hts test plan for Guest OS
  echo "echo 'sleep 10'                           >> $VT_SH"  >> $KS_FILE
  echo "echo 'cd /root/'                          >> $VT_SH"  >> $KS_FILE
  echo "echo 'hts discover'                       >> $VT_SH"  >> $KS_FILE
  echo "echo 'hts plan'                           >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test audio    --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test cdrom    --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test dummy    --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test floppy   --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test socket   --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test tape     --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test usb      --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test video    --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test virt     --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts plan --test z90crypt --disable' >> $VT_SH"  >> $KS_FILE 
  # TODO: network is not available now  
  echo "echo 'hts plan --test network  --disable' >> $VT_SH"  >> $KS_FILE 
  echo "echo 'hts run'                            >> $VT_SH"  >> $KS_FILE
  echo "echo 'hts submit'                         >> $VT_SH"  >> $KS_FILE
  echo "echo 'cp $RESULT_RPM /export/result/'     >> $VT_SH"  >> $KS_FILE
  echo "echo 'sleep 10'                           >> $VT_SH"  >> $KS_FILE
  echo "echo 'poweroff'                           >> $VT_SH"  >> $KS_FILE
  # run vtsh after the Guest reboots :-)
  echo "ln -s $VT_SH /etc/rc5.d/S99virt"                      >> $KS_FILE
  echo "ln -s $VT_SH /etc/rc3.d/S99virt"                      >> $KS_FILE
}

# create the bootable iso
create_iso() {
  clear
  echo ""
  echo "  Following RHEL5 install media types are accepted: "
  echo "    o RHEL5 Server DVD iso image file "
  echo "    o RHEL5 Server OS install file tree on the local disk "
  echo ""
  echo "  Press '1' if your install media is DVD iso image or "
  echo "  press '2' if your install media is OS install file tree "
  echo "  Followed by the ENTER key ... "
  read MEDIA_CHOICE whatever_else

  if [ $MEDIA_CHOICE == "1" ]; then
    clear
    echo ""
    echo "  Where is the RHEL5 install DVD iso file ? "
    echo "  Please input the absolute dir , starting with '/' : "
    echo "  (A valid dir should looks like : /tmp/RHEL5-Server-20061111.0-i386-DVD.iso ) "
    echo "  Input your dir and press ENTER to continue ... "
    read ISO_DIR whatever_else
    mount -o loop $ISO_DIR $INSTALL_MEDIA
  elif [ $MEDIA_CHOICE == "2"  ]; then
    clear
    echo ""
    echo "  Where is the RHEL5 OS install file tree ? "
    echo "  Please input the absolute dir , starting with '/' : "
    echo "  (A valid dir should looks like : /tmp/RHEL5-Server/i386 ) "
    echo "  Input your dir and press ENTER to continue ... "
    read OS_DIR whatever_else
    rm -rf $INSTALL_MEDIA
    ln -s $OS_DIR $INSTALL_MEDIA
  else
    echo "  Error ! No supported install media found ! "
    echo "  Please restart this test when the install media is ready ! "
    return 1
  fi

  # check whether the install_media is the right ARCH
  HOST_ARCH=`arch | tr -d [3-6] `
  GUEST_ARCH=`cat $INSTALL_MEDIA/.discinfo | head -n 3 | tail -n 1 | tr -d [3-6] `
  if [ $HOST_ARCH == $GUEST_ARCH ]; then
    echo "  The install media you provided is match with Host OS !  "
    echo "  Will continue in 3 seconds ... "
    sleep 3
  else  
    echo "  Error ! The install media you provided is not match with Host OS !  "
    echo "  Please make sure the install media is the same ARCH as Host OS,  "
    echo "  and restart this test !  "
    return 1
  fi

  mkdir $VIRT_DIR/newiso
  cp -rf $INSTALL_MEDIA/isolinux $VIRT_DIR/newiso/
  CFG_FILE=$VIRT_DIR/newiso/isolinux/isolinux.cfg
  chmod +w $CFG_FILE
  echo "default linux"                                        > $CFG_FILE
  echo "prompt 1"                                            >> $CFG_FILE
  echo "timeout 010"                                         >> $CFG_FILE
  echo "label linux"                                         >> $CFG_FILE
  echo "  kernel vmlinuz"                                    >> $CFG_FILE
  echo "  append ks=nfs:$HOST_IP:$KS_FILE initrd=initrd.img" >> $CFG_FILE
  chmod -w $CFG_FILE
  cd $VIRT_DIR/newiso/
  mkisofs -o $IMAGE_DIR/yk-virt.iso -b isolinux/isolinux.bin -c isolinux/boot.cat \
   -no-emul-boot -boot-load-size 4 -boot-info-table $VIRT_DIR/newiso/
}

# copy required four rpm packages to $EXPORT_RPM
copy_rpm() {
  clear
  echo ""
  echo "  Please specify where are the HTS related rpms, those you have installed "  
  echo "  on the host machine: "
  echo "        -- hts     "
  echo "        -- lmbench "
  echo "        -- stress  "
  echo "        -- dt      "
  echo ""
  for i in hts lmbench stress dt
  do
    echo ""
    echo "  Where is the '$i' install rpm package ? "
    echo "  Please input the absolute dir , starting with '/' : "
    echo "  (A valid dir should looks like : /tmp/$i-VERSION-RELEASE.ARCH.rpm ) "
    echo "  Input the dir and press ENTER to continue ... "
    read RPM_DIR whatever_else
    # check the package's name
    PACKAGE_NAME=`rpm -qp --qf '%{Name}\n' $RPM_DIR`
    if [ $PACKAGE_NAME == $i ]; then
      echo " package name match ! "
    else
      echo " Error ! The rpm file you specified is not correct ! "
      echo " Please check the package again and restart this test ! "
      return 1
    fi
    # check the package's ARCH
    SYSTEM_ARCH=`arch`
    if [ $SYSTEM_ARCH == i386 ] || [ $SYSTEM_ARCH == i486 ] || [ $SYSTEM_ARCH == i586 ] || [ $SYSTEM_ARCH == i686 ]; then
      SYSTEM_ARCH=i386
      echo " SYSTEM_ARCH is $SYSTEM_ARCH "
    fi
    if [ $i == hts ]; then
      echo " handling hts package... "
    elif grep $SYSTEM_ARCH $RPM_DIR ; then
      echo " package arch match ! "
    else
      echo " Error ! Package arch does not match ! "
      echo " Please check the package again and restart this test ! "
      return 1
    fi
    # copy the package to $EXPORT_RPM
    cp -f $RPM_DIR $EXPORT_RPM/
  done
}

# create the FV Guest OS and let it run
create_guest_fv() {
  echo "import os, re"                                              > $GUEST1_CFG
  echo "arch = os.uname()[4]"                                      >> $GUEST1_CFG
  echo "if re.search('64', arch):"                                 >> $GUEST1_CFG
  echo "    arch_libdir = 'lib64'"                                 >> $GUEST1_CFG
  echo "else:"                                                     >> $GUEST1_CFG
  echo "    arch_libdir = 'lib'"                                   >> $GUEST1_CFG
  echo "kernel = '/usr/lib/xen/boot/hvmloader'"                    >> $GUEST1_CFG
  echo "builder='hvm'"                                             >> $GUEST1_CFG
  echo "memory = $GUEST_MEM"                                       >> $GUEST1_CFG
  echo "name = '$GUEST1_NAME'"                                     >> $GUEST1_CFG
  # TODO: RHEL5 beta2 has some trouble of handling more than 1 vcpus
  #echo "vcpus=$GUEST1_CPU_NUM"                                     >> $GUEST1_CFG
  echo "vcpus=1"                                                   >> $GUEST1_CFG
  echo "pae=1"                                                     >> $GUEST1_CFG
  echo "acpi=1"                                                    >> $GUEST1_CFG
  echo "apic=1"                                                    >> $GUEST1_CFG
  # find xenbrX on multiple-NIC-machine 
  echo "vif = [ 'type=ioemu, bridge=$XENBR_X' ]"                   >> $GUEST1_CFG
  echo "disk = [ 'file:$IMAGE_DIR/virt1.img,hda,w', 'file:$IMAGE_DIR/yk-virt.iso,hdc:cdrom,r' ]" >> $GUEST1_CFG
  echo "device_model = '/usr/' + arch_libdir + '/xen/bin/qemu-dm'" >> $GUEST1_CFG
  echo "boot='d'"                                                  >> $GUEST1_CFG
  echo "vnc=1"                                                     >> $GUEST1_CFG
  echo "serial=\"pty\""                                            >> $GUEST1_CFG
  echo "on_poweroff = 'destroy'"                                   >> $GUEST1_CFG
  # set on_reboot to "destroy" is important ! otherwise the Guest will not halt
  echo "on_reboot   = 'destroy'"                                   >> $GUEST1_CFG
  echo "on_crash    = 'destroy'"                                   >> $GUEST1_CFG
  # create the Guest and wait it to finish
  /usr/sbin/xm create $GUEST1_CFG
}

# remove the /tmp/virt if it exists
remove_dir() {
  echo "The directory /tmp/virt exists, will remove it now ."
  umount -l /tmp/virt/install_media 2>/dev/null
  rm -rf /tmp/virt
}

# main function
virt_test() {
  # gather system info first
  echo ""
  echo " ------xm-info-start------ "
  /usr/sbin/xm info
  echo " ------xm-info-end------ "
  echo ""
  echo " ------xm-dmesg-start------ "
  /usr/sbin/xm dmesg
  echo " ------xm-dmesg-end------ "
  echo ""

  # turn off the firewall
  /etc/init.d/iptables stop

  # set Guest's name
  HOSTNAME=`hostname`
  GUEST1_NAME=$HOSTNAME-virt1
  GUEST1_CFG=/etc/xen/$GUEST1_NAME
  
  # remove the GUEST1_CFG if it exists
  rm -rf $GUEST1_CFG 2>/dev/null

  # TODO: find host's IP and NIC is ugly below
  # find the host's effective IP addr and NIC name
  for TMP_NIC1 in `ifconfig | grep Ethernet | awk '{print $1}'`
  do
    for TMP_NIC2 in `cat /etc/sysconfig/hwconf | grep device | awk '{print $2}'`
    do
      if [ $TMP_NIC1 == $TMP_NIC2 ]; then
        NIC=$TMP_NIC1
      fi
    done
  done
  HOST_IP=`ifconfig $NIC | grep addr | grep Bcast | awk '{print $2}' | tr -d [a-z:]`

  # let the Guest OS has the same num of CPUs as Host OS
  HOST_CPU_NUM=`cat /proc/cpuinfo | grep processor | wc -l`
  GUEST1_CPU_NUM=$HOST_CPU_NUM

  # specify half of the system's memory to Guest OS
  # but GUEST_MEM will not exceed 2GB and will not smaller than 512MB
  # TODO: is 2GB big enough ?
  MEM_TMP=`cat /proc/meminfo | head -n 1 | awk '{print $2}'`
  # the memory on test machine should be more than 1GB
  HOST_MEM=`expr $MEM_TMP / 1024`
  if [ $HOST_MEM -lt 1000 ]; then
    clear
    echo ""
    echo "  ----Warning !---- "
    echo "  This machine has only $HOST_MEM MiB of available memory ! "
    echo "  The minimum memory required to run this test is 1GiB . "
    echo ""
    echo "  However, you can still choose to continue. But the virt test may fail. "
    echo "  Please be aware of the following note in the RHEL5 Beta2 release notes: "
    echo "    o  Note that the minimum RAM required to install Red Hat Enterprise "
    echo "       Linux 4.92 has been raised to 1GB; the recommended RAM is 2GB. If "
    echo "       a machine has less than 1GB RAM, the installation process may hang. "
    echo ""
    echo "  If the test machine does have more than 1GiB of physical memory and you "
    echo "  have run the virt test several times since the system started, then "
    echo "  it's normal you see this message. Because virt test will 'consume' "
    echo "  system's memory. So please reboot your test machine and restart this "
    echo "  test ! "
    echo ""
    echo ""
    echo "  Press ENTER to continue or press Ctrl+C to quit . "
    read
    echo "  ... "
    echo "  Continue to run with $HOST_MEM MiB of memory ... "
    sleep 5
  fi
  GUEST_MEM=`expr $MEM_TMP / 2048`
  if [ $GUEST_MEM -gt 2048 ]; then
    GUEST_MEM=2048
  elif [ $GUEST_MEM -lt 512 ]; then
    GUEST_MEM=512
  fi  

  # remove the dir "/tmp/virt" if it exists
  VIRT_EXIST=`ls /tmp/virt 2>/dev/null | head -n 1`
  ${VIRT_EXIST:+remove_dir}

  # define tmp dirs and files here
  VIRT_DIR=/tmp/virt                    ;   mkdir --mode=777 $VIRT_DIR
  IMAGE_DIR=/var/lib/xen/images         ;   mkdir -p $IMAGE_DIR
  INSTALL_MEDIA=$VIRT_DIR/install_media ;   mkdir $INSTALL_MEDIA
  VIRT_TMP=$VIRT_DIR/tmp                ;   mkdir $VIRT_TMP
  EXPORT=$VIRT_DIR/export               ;   mkdir $EXPORT
  EXPORT_RPM=$EXPORT/rpm                ;   mkdir $EXPORT_RPM
  EXPORT_RESULT=$EXPORT/result          ;   mkdir $EXPORT_RESULT
  KS_DIR=$VIRT_DIR/ks                   ;   mkdir $KS_DIR
  KS_FILE=$KS_DIR/ks.cfg                ;   touch $KS_FILE

  # to find the xenbrX , supporting multiple NICs
  XENBR_X=`ifconfig | grep xenbr | awk '{print $1}'`
  
  # following variables are used by Guest OS
  VT_SH=/etc/init.d/vtsh
  RESULT_RPM=/usr/src/redhat/RPMS/noarch/hts*.noarch.rpm 

  # check arch
  ARCH=`arch`
  if [ $ARCH == x86_64 ] || [ $ARCH == i386 ] || [ $ARCH == i486 ] || [ $ARCH == i586 ] || [ $ARCH == i686 ]; then
    echo "  This machine's ARCH is $ARCH"
    sleep 5
  else
    echo "  Error ! This machine's ARCH is $ARCH "
    echo "  Virtualization feature is not available for this ARCH yet ! "
    sleep 3
    return 1
  fi

  # check PAE
  PAE=no
  for i in `cat /proc/cpuinfo`
  do
    if [ $i == pae ]; then
      PAE=yes
    fi
  done
  if [ $PAE == yes ]; then
    echo "  This machine has PAE "
  else
    echo "  Error ! This machine has no PAE support ! "
    return 1
  fi

  # check Intel-VT/AMD-V
  VIRT_SUPPORT=none
  for i in `cat /proc/cpuinfo`
  do
    if [ $i == vmx ]; then
      VIRT_SUPPORT=vmx
    elif [ $i == svm ]; then
      VIRT_SUPPORT=svm
    fi
  done
  if [ $VIRT_SUPPORT == vmx ]; then
    echo "  This machine has Intel's hardware support for virtualization : $VIRT_SUPPORT "
    PV_FV=fv
    sleep 5
  elif [ $VIRT_SUPPORT == svm ]; then
    echo " This machine has AMD's hardware support for virtualization : $VIRT_SUPPORT "
    PV_FV=fv
    sleep 5
  else
    echo " Error ! This machine does not have hardware support for virtualization ! "
    echo " The virt test will not run, exiting.... "
    sleep 5
    return 1
  fi  

  # check kernel
  XEN_KERNEL=`uname -a | grep xen | awk '{print $3}'`
  if [ -z $XEN_KERNEL ]; then
    echo "  Error ! You are running on a non-Xen kernel now ! "
    echo "  Please reboot to the right Xen kernel ! "
    return 1
  fi

  # check if Intel-VT/AMD-V is enabled
  if [ $PV_FV == fv ]; then
    VT_ENABLE=no
    VT_STATUS=`cat /sys/hypervisor/properties/capabilities| grep hvm| awk '{print $2}'`
    if [ -n $VT_STATUS ]; then
      VT_ENABLE=yes
      echo "  Hardware support for virtualization is enabled, start testing... "
    else
      echo "  Error ! The hardware support for virtualization is not enabled ! "
      echo "  Please reboot this machine and enable the virtualization feature "
      echo "  in the BIOS and restart this test ! "
      return 1
    fi
  fi

  # run create_ks_file
  if create_ks_file; then
    echo "  kickstart file successfully created ! "
    sleep 3
  else
    echo " create kickstart file error ! "
    return 1
  fi

  # run create_iso
  if create_iso; then
    echo "  boot iso successfully created ! "
  else
    echo "  create boot iso error ! "
    return 1
  fi

  # run copy_rpm
  if copy_rpm; then
    echo "  required rpms are successfully copied ! "
  else
    echo "  copy required rpm package error ! "
    return 1
  fi

  # run create_hd
  if create_hd; then
    echo "  Guest hd successfully created ! "
  else
    echo "  create Guest hd error ! "
    return 1
  fi

  # export nfs dir here
  service nfs stop 2>/dev/null
  echo "$EXPORT	*(rw,no_root_squash,sync)"         >> /etc/exports
  echo "$KS_DIR	*(rw,no_root_squash,sync)"         >> /etc/exports
  echo "$INSTALL_MEDIA	*(rw,no_root_squash,sync)" >> /etc/exports
  mv /etc/exports /etc/exports.bak
  cat /etc/exports.bak | sort -u > /etc/exports
  rm -rf /etc/exports.bak
  service nfs start
  exportfs -r

  # give user a hint before running the test :-)
  clear
  echo ""
  echo "  This test includes 2 stages and will take many hours to finish ! "
  echo "  Please relax and be patient :-) "
  echo ""
  echo ""
  sleep 2
  TIME_pv=30
  TIME_fv=120
  echo "  It'll cost at most $[TIME_$PV_FV] minites to create the $PV_FV Guest... "
  echo ""
  echo ""
  sleep 5
  CREATE_START_TIME=`date +%s`
  create_guest_$PV_FV &

  # check if Guest OS is created
  # creata a FV guest should not cost more than 2 hours
  WAIT_TIME_fv=7200
  VIRT_TRUE1=0
  while [ $VIRT_TRUE1 -lt 1 ]; do 
    sleep 300
    echo ""
    FINISHED=`/usr/sbin/xm list | grep $GUEST1_NAME | awk '{print $1}'`
    if [ -z $FINISHED ]; then
      echo "  Guest OS is successfully created !! "
      sleep 5
      VIRT_TRUE1=5
    else
      CREATE_END_TIME=`date +%s`
      CREATE_TIME=`expr $CREATE_END_TIME - $CREATE_START_TIME`
      if [ $CREATE_TIME -gt $[WAIT_TIME_$PV_FV] ]; then
        # create guest error !!
        echo " ---- Create $PV_FV guest Error ! ---- "
        /usr/sbin/xm destroy $GUEST1_NAME
        VIRT_TRUE1=5
        return 1
      else
        echo "  Virtualization test stage 1 is running...Please be patient... "
        echo "  Creating Guest OS... "
        echo "  Time elapsed: $CREATE_TIME seconds. "
      fi
    fi
  done

  # for FV guest, need to change 'd' to 'c' for "boot=" to boot from disk
  if [ $PV_FV == "fv" ]; then
    sleep 10
    mv $GUEST1_CFG /etc/xen/virt_tmp_cfg
    sed "s/boot='d'/boot='c'/g" /etc/xen/virt_tmp_cfg > $GUEST1_CFG
  fi

  # let user know the guest is up and running
  clear
  echo ""
  echo "  Guest OS is up and running ! "
  echo ""
  echo "  It'll cost at most 4 hours to finish... "
  echo ""

  # restart the Guest to run hts tests
  RUN_START_TIME=`date +%s`
  /usr/sbin/xm create $GUEST1_CFG

  # check if Guest OS is done
  # the running time for Guest OS should not cost more than 4 hours
  RUN_WAIT_TIME=14400
  VIRT_TRUE2=0
  while [ $VIRT_TRUE2 -lt 1 ]; do 
    sleep 300
    echo ""
    FINISHED=`/usr/sbin/xm list | grep $GUEST1_NAME | awk '{print $1}'`
    if [ -z $FINISHED ]; then
      echo "  Virtualization test finished !! "
      sleep 5
      VIRT_TRUE2=5
    else
      RUN_END_TIME=`date +%s`
      RUN_TIME=`expr $RUN_END_TIME - $RUN_START_TIME`
      if [ $RUN_TIME -gt $RUN_WAIT_TIME ]; then
        # something wrong while guest running
        echo " $PV_FV guest run Error ! "
        /usr/sbin/xm destroy $GUEST1_NAME
        VIRT_TRUE2=5
        return 1
      else
        echo "  Virtualization test stage 2 is running...Please be patient... "
        echo "  Guest OS is running... "
        echo "  Time elapsed: $RUN_TIME seconds. "
      fi
    fi
  done
  
  # check hts result which Guest OS generated
  rm -rf $VIRT_TMP/*
  cp $EXPORT_RESULT/hts*.noarch.rpm $VIRT_TMP/
  cd $VIRT_TMP/
  if [ `ls $VIRT_TMP/* | wc -l` -lt 1 ]; then
    echo "  Unexpected Error ! No result rpm present ! "
    return 1
  elif [ `ls $VIRT_TMP/* | wc -l` -gt 1 ]; then
    echo "  Unexpected Error ! Multiple result rpm files ! "
    return 1
  fi
  rpm2cpio ./hts*.noarch.rpm > ./result.cpio
  cpio -icdv < ./result.cpio
  GUEST1_FAIL=`grep -r FAIL ./var | wc -l `
  if [ $GUEST1_FAIL == 0 ]; then
    echo "  Virtualization test PASS on FV guest ! "
    sleep 5
  elif [ $GUEST1_FAIL == 1 ]; then
    if [ $PV_FV == "pv" ]; then
      echo "  Virtualization test PASS on PV guest ! "
      sleep 5
    else
      echo "  Virtualization test FAIL ! "
      return 1
    fi
  else
    echo "  Virtualization test FAIL ! "
    echo "  Some test(s) failed in the Guest OS ! "
    return 1
  fi
  cd /root/

  # restore /etc/exports
  umount $INSTALL_MEDIA
  mv /etc/exports /etc/exports.old
  sed "/\/tmp\/virt\//d" /etc/exports.old > /etc/exports
  rm -rf /etc/exports.old
  service nfs stop
  service nfs start

  # move guestOS's result rpm to hostOS's /usr/src/redhat/RPMS/noarch/
  # and keep only one guestOS's result
  mv -f $EXPORT_RESULT/hts*.noarch.rpm /usr/src/redhat/RPMS/noarch/
 
  # remove Guest OS and any tmp files
  rm -rf $VIRT_DIR
  rm -rf $IMAGE_DIR/virt1.img
  rm -rf $IMAGE_DIR/yk-virt.iso

  # gather xend.log and xend-debug.log info
  echo ""
  echo " ------xend.log-start------ "
  /usr/sbin/xm log
  echo " ------xend.log-end------ "
  echo ""
  echo " ------xend-debug.log-start------ "
  cat /var/log/xen/xend-debug.log
  echo " ------xend-debug.log-end------ "
  echo ""

  # restore the firewall
  /etc/init.d/iptables start
}

# run the test
virt_test


