#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Ported by : YangKun <ykun@redhat.com>
#
# Quick butcher from floppy.sh : Rob Landry <rlandry@redhat.com>
#


z90crypt_setup(){
 ( set -e
   lsmod | grep -q z90crypt || modprobe z90crypt
   grep -q pkcs11 /etc/group
   mkdir -p ./pkcs11/lite
   chown root:pkcs11 ./pkcs11
   # this allows us to start with the default PIN etc. and not overwrite
   # or remove any real PINs etc.
   mount --bind ./pkcs11 /etc/pkcs11
   service pkcsslotd start
 ) || echo "Unable to cleanly start crypto service"
   # We used to need this to avoid some timing issue; we might not now?
   sleep 2
}

z90crypt_shutdown(){
   umount /etc/pkcs11 2>/dev/null
   service pkcsslotd stop 2>/dev/null
}

z90crypt_test() {
    z90crypt_shutdown
    z90crypt_setup

    pin=87654321
    newpin=12345678
    userpin=43215678
    newuserpin=87651234
    methods=/usr/lib/pkcs11/methods
    pkcsconf=$methods/pkcsconf
    tokenlabel="RHRTOKEN"
    tokenlabel2="RHRTOKEN2"

    # unfortunately pkcsonf's use of exit codes is inconsistant
    # we'll hard code a number for now; but I believe it is
    # consistant within a release
    pkcsconf_exitcode=192

    # ---------- General Info ------------
    if [ `$pkcsconf -i 1>&2; echo $?` -eq $pkcsconf_exitcode ]; then true; else false; fi
    if [ `$pkcsconf -s 1>&2; echo $?` -eq $pkcsconf_exitcode ]; then true; else false; fi
    if [ `$pkcsconf -m 1>&2; echo $?` -eq $pkcsconf_exitcode ]; then true; else false; fi
    # ---------- Changing Pins ------------
    $pkcsconf -P -c 0 -S $pin -n $newpin
    $pkcsconf -u -c 0 -S $newpin -n $userpin
    $pkcsconf -p -c 0 -U $userpin -n $newuserpin
    echo $tokenlabel | $pkcsconf -I -c 0 -S $newpin
    [ `$pkcsconf -t | awk ' /Label:/ { print $2; } '` = $tokenlabel ]
    if [ `$pkcsconf -t 1>&2; echo $?` -eq $pkcsconf_exitcode ]; then true; else false; fi
    # "---------- Changing Token ------------"
    echo $tokenlabel2 | $pkcsconf -I -c 0 -S $newpin
    [ `$pkcsconf -t | awk ' /Label:/ { print $2; } '` = $tokenlabel2 ]
    if [ `$pkcsconf -t 1>&2; echo $?` -eq $pkcsconf_exitcode ]; then true; else false; fi
    # "---------- Non-root user -------------"
    echo "if [ `$pkcsconf -t` 1>&2; echo $? -eq $pkcsconf_exitcode ]; then true; else false; fi" | su - rhr2test -s /bin/bash

    z90crypt_shutdown
}

# run the test 
z90crypt_test


